function [finished ] = predict_kalman(data_ca, C_ca)



% convert the cell arrays from java to matrix 

nrow = length(data_ca);
data_all = [];
C_all = [];
for i = 1:nrow
	data_all = [data_all ; cell2mat(data_ca{i})];
    C_all = [C_all ; cell2mat(C_ca{i})];
end

if length(find(data_all > 100)) > 0
    fprintf('need to impute gene expression levels');
end


% first select a subset of genes which have GO annotations

annotatedgenes = find(sum(C_all > 0,2));
cutoff = 0.1;
while length(annotatedgenes) > 1000
    annotatedgenes = find(sum(C_all > cutoff,2));
    cutoff  =  cutoff + 0.1;
end
data = data_all(annotatedgenes,:);
C = C_all(annotatedgenes,:);

fprintf('inferring the underlying processes for %d genes.\n' , size(data,1));


finished = '0';
N = 1;
ss = size(C,2);
os = size(C,1);
Q = zeros(ss,ss,N);
initx = zeros(ss,N);
for d = 1:N
    Q(:,:,d) = 0.01*eye(ss);
    R(d) = 0.01;
    B(d) = 0.015;
    initV(:,:,d) = eye(ss);
end
diagQ = 1;
diagR = 1;
ARmode = 0;
help = 10 * ones(N,1);
selectedprocess = zeros(N,ss);
for j = 1:ss
        selectedprocess(:,j) = j;
end
full = 0;

ct(1) = 0;
ct(2) = size(data,2);



alpha = zeros(os, os, N);
Tsum = zeros(N,1);
for d = 1:N
  %y = data(:,:,ex);
  y = data(:,ct(d)+1:ct(d+1));
  T = size(y,2);
  Tsum(d) = T;
  alpha_temp = zeros(os, os);
  for t=1:T
    alpha_temp = alpha_temp + y(:,t)*y(:,t)';
  end
  alpha(:,:,d) = alpha(:,:,d) + alpha_temp;
end

numprocess = size(selectedprocess,2);
% Convert to inline function as needed.
%if ~isempty(constr_fun)
%  constr_fun = fcnchk(constr_fun,length(varargin));
%end

noedge = find(sum(C,2) == 0);

delta = zeros(os, ss,N);
gamma = zeros(ss, ss,N);
gamma1 = zeros(ss, ss,N);
gamma2 = zeros(ss, ss,N);
beta = zeros(ss, ss,N);
P1sum = zeros(ss, ss,N);
x1sum = zeros(ss, 1,N);
loglik = 0;
xsmooth = zeros(ss,ct(length(ct)));
%C = sparse(C);

%  for ex = 1:N
%    y = data{ex};
for d = 1:N
    y = data(:,ct(d)+1:ct(d+1));
    Qt = diag(Q(:,:,d));
    Qt = Qt(selectedprocess(d,:));
    Qt = diag(Qt);
    Vt = initV(selectedprocess(d,:),selectedprocess(d,:),d);
    Xt = initx(selectedprocess(d,:),d);
    tempR = repmat(R(d),os,1);
    tempR(noedge) = B(d);
    tempR = diag(tempR);
    [beta_t, gamma_t, delta_t, gamma1_t, gamma2_t, x1, V1, loglik_t, xsmooth(selectedprocess(d,:),ct(d)+1:ct(d+1)),help(d)] = ...
    Estep(y, eye(numprocess), C(:,selectedprocess(d,:)) , Qt, tempR, Xt, Vt, ARmode, 1,help(d));

%      result = sortrows([ sum(xsmooth(:,ct(d)+1:ct(d+1)).^2,2)  (1:108)'],1);
%      xsmooth(result(1:88,2),ct(d)+1:ct(d+1)) = 0 ; 

    beta(selectedprocess(d,:),selectedprocess(d,:),d) = beta_t;
%       gamma = gamma + gamma_t;
    delta(:,selectedprocess(d,:),d) = delta_t;
    gamma1(selectedprocess(d,:),selectedprocess(d,:),d) = gamma1_t;
    gamma2(selectedprocess(d,:),selectedprocess(d,:),d) = gamma2_t;
    P1sum(selectedprocess(d,:),selectedprocess(d,:),d) = V1 + x1*x1';
    x1sum(selectedprocess(d,:),d) = x1;
    %fprintf(1, 'example %d, ll/T %5.3f\n', ex, loglik_t/T);
    loglik = loglik + loglik_t;
end

dlmwrite('predicted_xsmooth',xsmooth,'\t');

finished = '1';


%%%%%%%%%

function [beta, gamma, delta, gamma1, gamma2, x1, V1, loglik, xsmooth,help] = ...
    Estep(y, A, C, Q, R, initx, initV, ARmode , num_iter,help)
%
% Compute the (expected) sufficient statistics for a single Kalman filter sequence.
%

[os T] = size(y);
ss = length(A);

if ARmode
  xsmooth = y;
  Vsmooth = zeros(ss, ss, T); % no uncertainty about the hidden states
  VVsmooth = zeros(ss, ss, T);
  loglik = 0;
else
  [xsmooth, Vsmooth, VVsmooth, loglik,help] = kalman_smoother(y, A, C, Q, R, initx, initV, num_iter,help);
end

delta = zeros(os, ss);
gamma = zeros(ss, ss);
beta = zeros(ss, ss);
for t=1:T
  delta = delta + y(:,t)*xsmooth(:,t)';
  gamma = gamma + xsmooth(:,t)*xsmooth(:,t)' + Vsmooth(:,:,t);
  if t>1 beta = beta + xsmooth(:,t)*xsmooth(:,t-1)' + VVsmooth(:,:,t); end
end
gamma1 = gamma - xsmooth(:,T)*xsmooth(:,T)' - Vsmooth(:,:,T);
gamma2 = gamma - xsmooth(:,1)*xsmooth(:,1)' - Vsmooth(:,:,1);

x1 = xsmooth(:,1);
V1 = Vsmooth(:,:,1);

